@@ -26,9 +26,7 @@
 #include "qwaylandhardwareintegration_p.h"
 #include "qwaylandtextinputv1_p.h"
 #include "qwaylandtextinputv2_p.h"
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-#include "qwaylandtextinputv4_p.h"
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+#include "qwaylandtextinputv3_p.h"
 #include "qwaylandinputcontext_p.h"
 #include "qwaylandinputmethodcontext_p.h"
 
@@ -47,9 +45,13 @@
 
 #include <QtWaylandClient/private/qwayland-text-input-unstable-v1.h>
 #include <QtWaylandClient/private/qwayland-text-input-unstable-v2.h>
-#include <QtWaylandClient/private/qwayland-text-input-unstable-v4-wip.h>
+#include <QtWaylandClient/private/qwayland-text-input-unstable-v3.h>
 #include <QtWaylandClient/private/qwayland-wp-primary-selection-unstable-v1.h>
 #include <QtWaylandClient/private/qwayland-qt-text-input-method-unstable-v1.h>
+#include <QtWaylandClient/private/qwayland-fractional-scale-v1.h>
+#include <QtWaylandClient/private/qwayland-viewporter.h>
+#include <QtWaylandClient/private/qwayland-cursor-shape-v1.h>
+#include <QtWaylandClient/private/qwayland-xdg-toplevel-drag-v1.h>
 
 #include <QtCore/private/qcore_unix_p.h>
 
@@ -63,18 +65,6 @@
 
 #include <tuple> // for std::tie
 
-static void checkWaylandError(struct wl_display *display)
-{
-    int ecode = wl_display_get_error(display);
-    if ((ecode == EPIPE || ecode == ECONNRESET)) {
-        // special case this to provide a nicer error
-        qWarning("The Wayland connection broke. Did the Wayland compositor die?");
-    } else {
-        qWarning("The Wayland connection experienced a fatal error: %s", strerror(ecode));
-    }
-    _exit(1);
-}
-
 QT_BEGIN_NAMESPACE
 
 namespace QtWaylandClient {
@@ -112,9 +102,13 @@ public:
          * not only the one issued from event thread's waitForReading(), which means functions
          * called from dispatch_pending() can safely spin an event loop.
          */
+        if (m_quitting)
+            return;
+
         for (;;) {
             if (dispatchQueuePending() < 0) {
-                checkWaylandError(m_wldisplay);
+                Q_EMIT waylandError();
+                m_quitting = true;
                 return;
             }
 
@@ -152,6 +146,7 @@ public:
 
 Q_SIGNALS:
     void needReadAndDispatch();
+    void waylandError();
 
 protected:
     void run() override
@@ -265,14 +260,14 @@ Q_LOGGING_CATEGORY(lcQpaWayland, "qt.qpa.wayland"); // for general (uncategorize
 
 struct wl_surface *QWaylandDisplay::createSurface(void *handle)
 {
-    struct wl_surface *surface = mCompositor.create_surface();
+    struct wl_surface *surface = mGlobals.compositor->create_surface();
     wl_surface_set_user_data(surface, handle);
     return surface;
 }
 
 struct ::wl_region *QWaylandDisplay::createRegion(const QRegion &qregion)
 {
-    struct ::wl_region *region = mCompositor.create_region();
+    struct ::wl_region *region = mGlobals.compositor->create_region();
 
     for (const QRect &rect : qregion)
         wl_region_add(region, rect.x(), rect.y(), rect.width(), rect.height());
@@ -282,7 +277,7 @@ struct ::wl_region *QWaylandDisplay::createRegion(const QRegion &qregion)
 
 ::wl_subsurface *QWaylandDisplay::createSubSurface(QWaylandWindow *window, QWaylandWindow *parent)
 {
-    if (!mSubCompositor) {
+    if (!mGlobals.subCompositor) {
         qCWarning(lcQpaWayland) << "Can't create subsurface, not supported by the compositor.";
         return nullptr;
     }
@@ -291,7 +286,18 @@ struct ::wl_region *QWaylandDisplay::createRegion(const QRegion &qregion)
     Q_ASSERT(parent->wlSurface());
     Q_ASSERT(window->wlSurface());
 
-    return mSubCompositor->get_subsurface(window->wlSurface(), parent->wlSurface());
+    return mGlobals.subCompositor->get_subsurface(window->wlSurface(), parent->wlSurface());
+}
+
+::wp_viewport *QWaylandDisplay::createViewport(QWaylandWindow *window)
+{
+    if (!mGlobals.viewporter) {
+        qCWarning(lcQpaWayland) << "Can't create wp_viewport, not supported by the compositor.";
+        return nullptr;
+    }
+
+    Q_ASSERT(window->wlSurface());
+    return mGlobals.viewporter->get_viewport(window->wlSurface());
 }
 
 QWaylandShellIntegration *QWaylandDisplay::shellIntegration() const
@@ -306,7 +312,7 @@ QWaylandClientBufferIntegration * QWaylandDisplay::clientBufferIntegration() con
 
 QWaylandWindowManagerIntegration *QWaylandDisplay::windowManagerIntegration() const
 {
-    return mWindowManagerIntegration.data();
+    return mGlobals.windowManagerIntegration.get();
 }
 
 QWaylandDisplay::QWaylandDisplay(QWaylandIntegration *waylandIntegration)
@@ -315,22 +321,26 @@ QWaylandDisplay::QWaylandDisplay(QWaylandIntegration *waylandIntegration)
     qRegisterMetaType<uint32_t>("uint32_t");
 
     mDisplay = wl_display_connect(nullptr);
-    if (!mDisplay) {
+    if (mDisplay) {
+        setupConnection();
+    } else {
         qErrnoWarning(errno, "Failed to create wl_display");
-        return;
     }
 
+    mWaylandTryReconnect = qEnvironmentVariableIsSet("QT_WAYLAND_RECONNECT");
+}
+
+void QWaylandDisplay::setupConnection()
+{
     struct ::wl_registry *registry = wl_display_get_registry(mDisplay);
     init(registry);
 
-    mWindowManagerIntegration.reset(new QWaylandWindowManagerIntegration(this));
-
 #if QT_CONFIG(xkbcommon)
     mXkbContext.reset(xkb_context_new(XKB_CONTEXT_NO_FLAGS));
     if (!mXkbContext)
         qCWarning(lcQpaWayland, "failed to create xkb context");
 #endif
-    if (!mClientSideInputContextRequested)
+    if (mWaylandInputContextRequested)
         checkTextInputProtocol();
 }
 
@@ -345,38 +355,49 @@ QWaylandDisplay::~QWaylandDisplay(void)
     if (mSyncCallback)
         wl_callback_destroy(mSyncCallback);
 
-    qDeleteAll(qExchange(mInputDevices, {}));
+    qDeleteAll(std::exchange(mInputDevices, {}));
 
-    for (QWaylandScreen *screen : qExchange(mScreens, {})) {
+    for (QWaylandScreen *screen : std::exchange(mScreens, {})) {
         QWindowSystemInterface::handleScreenRemoved(screen);
     }
     qDeleteAll(mWaitingScreens);
 
-#if QT_CONFIG(wayland_datadevice)
-    mDndSelectionHandler.reset();
-#endif
 #if QT_CONFIG(cursor)
     mCursorThemes.clear();
 #endif
-    if (mDisplay)
-        wl_display_disconnect(mDisplay);
 
     if (m_frameEventQueue)
         wl_event_queue_destroy(m_frameEventQueue);
+
+    // Reset the globals manually since they need to be destroyed before the wl_display
+    mGlobals = {};
+
+    if (object())
+        wl_registry_destroy(object());
+
+    if (mDisplay)
+        wl_display_disconnect(mDisplay);
 }
 
 // Steps which is called just after constructor. This separates registry_global() out of the constructor
 // so that factory functions in integration can be overridden.
-void QWaylandDisplay::initialize()
+bool QWaylandDisplay::initialize()
 {
+    if (!isInitialized())
+        return false;
+
     forceRoundTrip();
 
+    emit connected();
+
     if (!mWaitingScreens.isEmpty()) {
         // Give wl_output.done and zxdg_output_v1.done events a chance to arrive
         forceRoundTrip();
     }
-    if (!mClientSideInputContextRequested)
+    if (mWaylandInputContextRequested)
         mTextInputManagerIndex = INT_MAX;
+
+    return qEnvironmentVariableIntValue("QT_WAYLAND_DONT_CHECK_SHELL_INTEGRATION") || shellIntegration();
 }
 
 void QWaylandDisplay::ensureScreen()
@@ -384,14 +405,101 @@ void QWaylandDisplay::ensureScreen()
     if (!mScreens.empty() || mPlaceholderScreen)
         return; // There are real screens or we already have a fake one
 
-    qCInfo(lcQpaWayland) << "Creating a fake screen in order for Qt not to crash";
+    qCInfo(lcQpaWayland) << "There are no outputs - creating placeholder screen";
 
     mPlaceholderScreen = new QPlatformPlaceholderScreen();
     QWindowSystemInterface::handleScreenAdded(mPlaceholderScreen);
     Q_ASSERT(!QGuiApplication::screens().empty());
 }
 
-// Called in main thread, either from queued signal or directly.
+void QWaylandDisplay::reconnect()
+{
+    qCWarning(lcQpaWayland) << "Attempting wayland reconnect";
+    m_eventThread->stop();
+    m_frameEventQueueThread->stop();
+    m_eventThread->wait();
+    m_frameEventQueueThread->wait();
+
+    qDeleteAll(mWaitingScreens);
+    mWaitingScreens.clear();
+
+    while (!mScreens.isEmpty()) {
+        auto screen = mScreens.takeLast();
+        ensureScreen();
+        QWindowSystemInterface::handleScreenRemoved(screen);
+    }
+
+    mCursorThemes.clear();
+    mCursor.reset();
+
+    mGlobals = GlobalHolder();
+
+    mWaylandIntegration->reset();
+
+    qDeleteAll(std::exchange(mInputDevices, {}));
+    mLastInputDevice = nullptr;
+
+    for (const RegistryGlobal &global : mRegistryGlobals) {
+        emit globalRemoved(global);
+    }
+    mRegistryGlobals.clear();
+
+    mLastInputSerial = 0;
+    mLastInputWindow.clear();
+    mLastKeyboardFocus.clear();
+    mActiveWindows.clear();
+
+    const auto windows = QGuiApplication::allWindows();
+    for (auto window : windows) {
+        if (auto waylandWindow = static_cast<QWaylandWindow *>(window->handle()))
+            waylandWindow->closeChildPopups();
+    }
+    // Remove windows that do not need to be recreated and now closed popups
+    QList<QWaylandWindow *> recreateWindows;
+    for (auto window : std::as_const(windows)) {
+        auto waylandWindow = static_cast<QWaylandWindow*>(window->handle());
+        if (waylandWindow && waylandWindow->wlSurface()) {
+            waylandWindow->reset();
+            recreateWindows.push_back(waylandWindow);
+        }
+    }
+
+    if (mSyncCallback) {
+        wl_callback_destroy(mSyncCallback);
+        mSyncCallback = nullptr;
+    }
+
+    if (object())
+        wl_registry_destroy(object());
+    mDisplay = wl_display_connect(nullptr);
+    if (!mDisplay)
+        _exit(1);
+
+    setupConnection();
+    initialize();
+
+    if (m_frameEventQueue)
+        wl_event_queue_destroy(m_frameEventQueue);
+    initEventThread();
+
+    auto needsRecreate = [](QPlatformWindow *window) {
+        return window && !static_cast<QWaylandWindow *>(window)->wlSurface();
+    };
+    auto window = recreateWindows.begin();
+    while (!recreateWindows.isEmpty()) {
+        if (!needsRecreate((*window)->QPlatformWindow::parent()) && !needsRecreate((*window)->transientParent())) {
+            (*window)->reinit();
+            window = recreateWindows.erase(window);
+        } else {
+            ++window;
+        }
+        if (window == recreateWindows.end())
+            window = recreateWindows.begin();
+    }
+
+    mWaylandIntegration->reconfigureInputContext();
+}
+
 void QWaylandDisplay::flushRequests()
 {
     m_eventThread->readAndDispatchEvents();
@@ -406,6 +514,8 @@ void QWaylandDisplay::initEventThread()
             new EventThread(mDisplay, /* default queue */ nullptr, EventThread::EmitToDispatch));
     connect(m_eventThread.get(), &EventThread::needReadAndDispatch, this,
             &QWaylandDisplay::flushRequests, Qt::QueuedConnection);
+    connect(m_eventThread.get(), &EventThread::waylandError, this,
+            &QWaylandDisplay::checkWaylandError, Qt::QueuedConnection);
     m_eventThread->start();
 
     // wl_display_disconnect() free this.
@@ -415,25 +525,48 @@ void QWaylandDisplay::initEventThread()
     m_frameEventQueueThread->start();
 }
 
+void QWaylandDisplay::checkWaylandError()
+{
+    int ecode = wl_display_get_error(mDisplay);
+    if ((ecode == EPIPE || ecode == ECONNRESET)) {
+        qWarning("The Wayland connection broke. Did the Wayland compositor die?");
+        if (mWaylandTryReconnect) {
+            reconnect();
+            return;
+        }
+    } else {
+        qWarning("The Wayland connection experienced a fatal error: %s", strerror(ecode));
+    }
+    _exit(-1);
+}
+
 void QWaylandDisplay::blockingReadEvents()
 {
-    if (wl_display_dispatch(mDisplay) < 0)
-        checkWaylandError(mDisplay);
+    if (wl_display_dispatch(mDisplay) < 0) {
+        int ecode = wl_display_get_error(mDisplay);
+        if ((ecode == EPIPE || ecode == ECONNRESET))
+            qWarning("The Wayland connection broke during blocking read event. Did the Wayland compositor die?");
+        else
+            qWarning("The Wayland connection experienced a fatal error during blocking read event: %s", strerror(ecode));
+        _exit(-1);
+    }
 }
 
 void QWaylandDisplay::checkTextInputProtocol()
 {
     QStringList tips, timps; // for text input protocols and text input manager protocols
+    // zwp_text_input_v2 is preferred over zwp_text_input_v3 because:
+    // - Currently, v3 is not as feature rich as v2.
+    // - While v2 is not upstreamed, it is well supported by KWin since Plasma 5 and Plasma
+    //   Mobile uses some v2 only.
     tips << QLatin1String(QtWayland::qt_text_input_method_v1::interface()->name)
          << QLatin1String(QtWayland::zwp_text_input_v2::interface()->name)
+         << QLatin1String(QtWayland::zwp_text_input_v3::interface()->name)
          << QLatin1String(QtWayland::zwp_text_input_v1::interface()->name);
     timps << QLatin1String(QtWayland::qt_text_input_method_manager_v1::interface()->name)
           << QLatin1String(QtWayland::zwp_text_input_manager_v2::interface()->name)
+          << QLatin1String(QtWayland::zwp_text_input_manager_v3::interface()->name)
           << QLatin1String(QtWayland::zwp_text_input_manager_v1::interface()->name);
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-    tips << QLatin1String(QtWayland::zwp_text_input_v4::interface()->name);
-    timps << QLatin1String(QtWayland::zwp_text_input_manager_v4::interface()->name);
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
 
     QString tiProtocols = QString::fromLocal8Bit(qgetenv("QT_WAYLAND_TEXT_INPUT_PROTOCOL"));
     qCDebug(lcQpaWayland) << "QT_WAYLAND_TEXT_INPUT_PROTOCOL=" << tiProtocols;
@@ -469,81 +602,115 @@ void QWaylandDisplay::handleScreenInitialized(QWaylandScreen *screen)
     mScreens.append(screen);
     QWindowSystemInterface::handleScreenAdded(screen);
     if (mPlaceholderScreen) {
-        QWindowSystemInterface::handleScreenRemoved(mPlaceholderScreen);
         // handleScreenRemoved deletes the platform screen
+        QPlatformScreen *s = mPlaceholderScreen;
         mPlaceholderScreen = nullptr;
+        QWindowSystemInterface::handleScreenRemoved(s);
+
     }
 }
 
+template <typename T, auto f>
+struct WithDestructor : public T
+{
+    using T::T;
+    ~WithDestructor()
+    {
+        f(this->object());
+    }
+};
+
 void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uint32_t version)
 {
     struct ::wl_registry *registry = object();
 
+    static QStringList interfaceBlacklist = qEnvironmentVariable("QT_WAYLAND_DISABLED_INTERFACES").split(u',');
+    if (interfaceBlacklist.contains(interface)) {
+        return;
+    }
+
     if (interface == QLatin1String(QtWayland::wl_output::interface()->name)) {
         mWaitingScreens << mWaylandIntegration->createPlatformScreen(this, version, id);
     } else if (interface == QLatin1String(QtWayland::wl_compositor::interface()->name)) {
-        mCompositor.init(registry, id, qMin((int)version, 4));
+        mGlobals.compositor.reset(
+                new WithDestructor<QtWayland::wl_compositor, wl_compositor_destroy>(
+                        registry, id, qMin((int)version, 6)));
     } else if (interface == QLatin1String(QWaylandShm::interface()->name)) {
-        mShm.reset(new QWaylandShm(this, version, id));
+        mGlobals.shm.reset(new QWaylandShm(this, version, id));
     } else if (interface == QLatin1String(QWaylandInputDevice::interface()->name)) {
         QWaylandInputDevice *inputDevice = mWaylandIntegration->createInputDevice(this, version, id);
         mInputDevices.append(inputDevice);
 #if QT_CONFIG(wayland_datadevice)
     } else if (interface == QLatin1String(QWaylandDataDeviceManager::interface()->name)) {
-        mDndSelectionHandler.reset(new QWaylandDataDeviceManager(this, version, id));
+        mGlobals.dndSelectionHandler.reset(new QWaylandDataDeviceManager(this, version, id));
 #endif
     } else if (interface == QLatin1String(QtWayland::qt_surface_extension::interface()->name)) {
-        mWindowExtension.reset(new QtWayland::qt_surface_extension(registry, id, 1));
+        mGlobals.surfaceExtension.reset(
+                new WithDestructor<QtWayland::qt_surface_extension, qt_surface_extension_destroy>(
+                        registry, id, 1));
     } else if (interface == QLatin1String(QtWayland::wl_subcompositor::interface()->name)) {
-        mSubCompositor.reset(new QtWayland::wl_subcompositor(registry, id, 1));
+        mGlobals.subCompositor.reset(
+                new WithDestructor<QtWayland::wl_subcompositor, wl_subcompositor_destroy>(registry,
+                                                                                          id, 1));
     } else if (interface == QLatin1String(QWaylandTouchExtension::interface()->name)) {
-        mTouchExtension.reset(new QWaylandTouchExtension(this, id));
+        mGlobals.touchExtension.reset(new QWaylandTouchExtension(this, id));
     } else if (interface == QLatin1String(QWaylandQtKeyExtension::interface()->name)) {
-        mQtKeyExtension.reset(new QWaylandQtKeyExtension(this, id));
+        mGlobals.qtKeyExtension.reset(new QWaylandQtKeyExtension(this, id));
 #if QT_CONFIG(tabletevent)
     } else if (interface == QLatin1String(QWaylandTabletManagerV2::interface()->name)) {
-        mTabletManager.reset(new QWaylandTabletManagerV2(this, id, qMin(1, int(version))));
+        mGlobals.tabletManager.reset(new QWaylandTabletManagerV2(this, id, qMin(1, int(version))));
+        for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
+            inputDevice->setTabletSeat(
+                    new QWaylandTabletSeatV2(mGlobals.tabletManager.get(), inputDevice));
 #endif
     } else if (interface == QLatin1String(QWaylandPointerGestures::interface()->name)) {
-        mPointerGestures.reset(new QWaylandPointerGestures(this, id, 1));
+        mGlobals.pointerGestures.reset(new QWaylandPointerGestures(this, id, 1));
 #if QT_CONFIG(wayland_client_primary_selection)
     } else if (interface == QLatin1String(QWaylandPrimarySelectionDeviceManagerV1::interface()->name)) {
-        mPrimarySelectionManager.reset(new QWaylandPrimarySelectionDeviceManagerV1(this, id, 1));
+        mGlobals.primarySelectionManager.reset(
+                new QWaylandPrimarySelectionDeviceManagerV1(this, id, 1));
+        for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
+            inputDevice->setPrimarySelectionDevice(
+                    mGlobals.primarySelectionManager->createDevice(inputDevice));
 #endif
     } else if (interface == QLatin1String(QtWayland::qt_text_input_method_manager_v1::interface()->name)
             && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
         qCDebug(lcQpaWayland) << "text input: register qt_text_input_method_manager_v1";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputManagerv1.reset();
-            mTextInputManagerv2.reset();
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            mTextInputManagerv4.reset();
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+            mGlobals.textInputManagerv1.reset();
+            mGlobals.textInputManagerv2.reset();
+            mGlobals.textInputManagerv3.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInput(nullptr);
         }
 
-        mTextInputMethodManager.reset(new QtWayland::qt_text_input_method_manager_v1(registry, id, 1));
+        mGlobals.textInputMethodManager.reset(
+                new WithDestructor<QtWayland::qt_text_input_method_manager_v1,
+                                   qt_text_input_method_manager_v1_destroy>(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setTextInputMethod(new QWaylandTextInputMethod(this, mTextInputMethodManager->get_text_input_method(inputDevice->wl_seat())));
+            inputDevice->setTextInputMethod(new QWaylandTextInputMethod(
+                    this,
+                    mGlobals.textInputMethodManager->get_text_input_method(
+                            inputDevice->wl_seat())));
         mWaylandIntegration->reconfigureInputContext();
         mTextInputManagerIndex = mTextInputManagerList.indexOf(interface);
     } else if (interface == QLatin1String(QtWayland::zwp_text_input_manager_v1::interface()->name)
                && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
         qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v1";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputMethodManager.reset();
-            mTextInputManagerv2.reset();
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            mTextInputManagerv4.reset();
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+            mGlobals.textInputMethodManager.reset();
+            mGlobals.textInputManagerv2.reset();
+            mGlobals.textInputManagerv3.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInputMethod(nullptr);
         }
 
-        mTextInputManagerv1.reset(new QtWayland::zwp_text_input_manager_v1(registry, id, 1));
+        mGlobals.textInputManagerv1.reset(
+                new WithDestructor<QtWayland::zwp_text_input_manager_v1,
+                                   zwp_text_input_manager_v1_destroy>(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices)) {
-            auto textInput = new QWaylandTextInputv1(this, mTextInputManagerv1->create_text_input());
+            auto textInput =
+                    new QWaylandTextInputv1(this, mGlobals.textInputManagerv1->create_text_input());
             textInput->setSeat(inputDevice->wl_seat());
             inputDevice->setTextInput(textInput);
         }
@@ -554,54 +721,75 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
             && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
         qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v2";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputMethodManager.reset();
-            mTextInputManagerv1.reset();
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            mTextInputManagerv4.reset();
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+            mGlobals.textInputMethodManager.reset();
+            mGlobals.textInputManagerv1.reset();
+            mGlobals.textInputManagerv3.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInputMethod(nullptr);
         }
 
-        mTextInputManagerv2.reset(new QtWayland::zwp_text_input_manager_v2(registry, id, 1));
+        mGlobals.textInputManagerv2.reset(
+                new WithDestructor<QtWayland::zwp_text_input_manager_v2,
+                                   zwp_text_input_manager_v2_destroy>(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setTextInput(new QWaylandTextInputv2(this, mTextInputManagerv2->get_text_input(inputDevice->wl_seat())));
+            inputDevice->setTextInput(new QWaylandTextInputv2(
+                    this, mGlobals.textInputManagerv2->get_text_input(inputDevice->wl_seat())));
         mWaylandIntegration->reconfigureInputContext();
         mTextInputManagerIndex = mTextInputManagerList.indexOf(interface);
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-    } else if (interface == QLatin1String(QtWayland::zwp_text_input_manager_v4::interface()->name)
+    } else if (interface == QLatin1String(QtWayland::zwp_text_input_manager_v3::interface()->name)
             && (mTextInputManagerList.contains(interface) && mTextInputManagerList.indexOf(interface) < mTextInputManagerIndex)) {
-        qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v4";
+        qCDebug(lcQpaWayland) << "text input: register zwp_text_input_v3";
         if (mTextInputManagerIndex < INT_MAX) {
-            mTextInputMethodManager.reset();
-            mTextInputManagerv2.reset();
+            mGlobals.textInputMethodManager.reset();
+            mGlobals.textInputManagerv2.reset();
             for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                 inputDevice->setTextInputMethod(nullptr);
         }
-
-        mTextInputManagerv4.reset(new QtWayland::zwp_text_input_manager_v4(registry, id, 1));
+        mGlobals.textInputManagerv3.reset(
+                new WithDestructor<QtWayland::zwp_text_input_manager_v3,
+                                   zwp_text_input_manager_v3_destroy>(registry, id, 1));
         for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
-            inputDevice->setTextInput(new QWaylandTextInputv4(this, mTextInputManagerv4->get_text_input(inputDevice->wl_seat())));
+            inputDevice->setTextInput(new QWaylandTextInputv3(
+                    this, mGlobals.textInputManagerv3->get_text_input(inputDevice->wl_seat())));
+
         mWaylandIntegration->reconfigureInputContext();
         mTextInputManagerIndex = mTextInputManagerList.indexOf(interface);
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
-    } else if (interface == QLatin1String(QWaylandHardwareIntegration::interface()->name)) {
+    }else if (interface == QLatin1String(QWaylandHardwareIntegration::interface()->name)) {
         bool disableHardwareIntegration = qEnvironmentVariableIntValue("QT_WAYLAND_DISABLE_HW_INTEGRATION");
         if (!disableHardwareIntegration) {
-            mHardwareIntegration.reset(new QWaylandHardwareIntegration(registry, id));
+            mGlobals.hardwareIntegration.reset(new QWaylandHardwareIntegration(registry, id));
             // make a roundtrip here since we need to receive the events sent by
             // qt_hardware_integration before creating windows
             forceRoundTrip();
         }
     } else if (interface == QLatin1String(QWaylandXdgOutputManagerV1::interface()->name)) {
-        mXdgOutputManager.reset(new QWaylandXdgOutputManagerV1(this, id, version));
+        mGlobals.xdgOutputManager.reset(new QWaylandXdgOutputManagerV1(this, id, version));
         for (auto *screen : std::as_const(mWaitingScreens))
             screen->initXdgOutput(xdgOutputManager());
-        forceRoundTrip();
+    } else if (interface == QLatin1String(QtWayland::wp_fractional_scale_manager_v1::interface()->name)) {
+        mGlobals.fractionalScaleManager.reset(
+                new WithDestructor<QtWayland::wp_fractional_scale_manager_v1,
+                                   wp_fractional_scale_manager_v1_destroy>(registry, id, 1));
+    } else if (interface == QLatin1String("wp_viewporter")) {
+        mGlobals.viewporter.reset(
+                new WithDestructor<QtWayland::wp_viewporter, wp_viewporter_destroy>(
+                        registry, id, qMin(1u, version)));
+    } else if (interface == QLatin1String(QtWayland::wp_cursor_shape_manager_v1::interface()->name)) {
+        mGlobals.cursorShapeManager.reset(new WithDestructor<QtWayland::wp_cursor_shape_manager_v1,
+                                                             wp_cursor_shape_manager_v1_destroy>(
+                registry, id, std::min(1u, version)));
+    } else if (
+            interface == QLatin1String(QtWayland::xdg_toplevel_drag_manager_v1::interface()->name)) {
+        mGlobals.xdgToplevelDragManager.reset(
+                new WithDestructor<QtWayland::xdg_toplevel_drag_manager_v1,
+                                   xdg_toplevel_drag_manager_v1_destroy>(registry, id, 1));
+    } else if (interface == QLatin1String(QtWayland::qt_windowmanager::interface()->name)) {
+        mGlobals.windowManagerIntegration.reset(
+                new QWaylandWindowManagerIntegration(this, id, version));
     }
 
-    mGlobals.append(RegistryGlobal(id, interface, version, registry));
-    emit globalAdded(mGlobals.back());
+    mRegistryGlobals.append(RegistryGlobal(id, interface, version, registry));
+    emit globalAdded(mRegistryGlobals.back());
 
     const auto copy = mRegistryListeners; // be prepared for listeners unregistering on notification
     for (Listener l : copy)
@@ -610,8 +798,8 @@ void QWaylandDisplay::registry_global(uint32_t id, const QString &interface, uin
 
 void QWaylandDisplay::registry_global_remove(uint32_t id)
 {
-    for (int i = 0, ie = mGlobals.size(); i != ie; ++i) {
-        RegistryGlobal &global = mGlobals[i];
+    for (int i = 0, ie = mRegistryGlobals.size(); i != ie; ++i) {
+        RegistryGlobal &global = mRegistryGlobals[i];
         if (global.id == id) {
             if (global.interface == QLatin1String(QtWayland::wl_output::interface()->name)) {
                 for (auto *screen : mWaitingScreens) {
@@ -633,32 +821,37 @@ void QWaylandDisplay::registry_global_remove(uint32_t id)
                 }
             }
             if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v1::interface()->name)) {
-                mTextInputManagerv1.reset();
+                mGlobals.textInputManagerv1.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInput(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
             if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v2::interface()->name)) {
-                mTextInputManagerv2.reset();
+                mGlobals.textInputManagerv2.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInput(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-            if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v4::interface()->name)) {
-                mTextInputManagerv4.reset();
+            if (global.interface == QLatin1String(QtWayland::zwp_text_input_manager_v3::interface()->name)) {
+                mGlobals.textInputManagerv3.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInput(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
             if (global.interface == QLatin1String(QtWayland::qt_text_input_method_manager_v1::interface()->name)) {
-                mTextInputMethodManager.reset();
+                mGlobals.textInputMethodManager.reset();
                 for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
                     inputDevice->setTextInputMethod(nullptr);
                 mWaylandIntegration->reconfigureInputContext();
             }
-            emit globalRemoved(mGlobals.takeAt(i));
+#if QT_CONFIG(wayland_client_primary_selection)
+            if (global.interface == QLatin1String(QtWayland::zwp_primary_selection_device_manager_v1::interface()->name)) {
+                mGlobals.primarySelectionManager.reset();
+                for (QWaylandInputDevice *inputDevice : std::as_const(mInputDevices))
+                    inputDevice->setPrimarySelectionDevice(nullptr);
+            }
+#endif
+            emit globalRemoved(mRegistryGlobals.takeAt(i));
             break;
         }
     }
@@ -666,7 +859,7 @@ void QWaylandDisplay::registry_global_remove(uint32_t id)
 
 bool QWaylandDisplay::hasRegistryGlobal(QStringView interfaceName) const
 {
-    for (const RegistryGlobal &global : mGlobals)
+    for (const RegistryGlobal &global : mRegistryGlobals)
         if (global.interface == interfaceName)
             return true;
 
@@ -677,8 +870,9 @@ void QWaylandDisplay::addRegistryListener(RegistryListener listener, void *data)
 {
     Listener l = { listener, data };
     mRegistryListeners.append(l);
-    for (int i = 0, ie = mGlobals.size(); i != ie; ++i)
-        (*l.listener)(l.data, mGlobals[i].registry, mGlobals[i].id, mGlobals[i].interface, mGlobals[i].version);
+    for (int i = 0, ie = mRegistryGlobals.size(); i != ie; ++i)
+        (*l.listener)(l.data, mRegistryGlobals[i].registry, mRegistryGlobals[i].id,
+                      mRegistryGlobals[i].interface, mRegistryGlobals[i].version);
 }
 
 void QWaylandDisplay::removeListener(RegistryListener listener, void *data)
@@ -689,16 +883,6 @@ void QWaylandDisplay::removeListener(RegistryListener listener, void *data)
     mRegistryListeners.erase(iter, mRegistryListeners.end());
 }
 
-uint32_t QWaylandDisplay::currentTimeMillisec()
-{
-    //### we throw away the time information
-    struct timeval tv;
-    int ret = gettimeofday(&tv, nullptr);
-    if (ret == 0)
-        return tv.tv_sec*1000 + tv.tv_usec/1000;
-    return 0;
-}
-
 void QWaylandDisplay::forceRoundTrip()
 {
      wl_display_roundtrip(mDisplay);
@@ -712,7 +896,12 @@ bool QWaylandDisplay::supportsWindowDecoration() const
     if (disabled)
         return false;
 
-    static bool integrationSupport = clientBufferIntegration() && clientBufferIntegration()->supportsWindowDecoration();
+    // Don't initialize client buffer integration just to check whether it can have a decoration.
+    if (!mWaylandIntegration->mClientBufferIntegrationInitialized)
+        return true;
+
+    // We can do software-rendered decorations, only disable them if the integration explicitly says it can't.
+    static bool integrationSupport = !clientBufferIntegration() || clientBufferIntegration()->supportsWindowDecoration();
     return integrationSupport;
 }
 
@@ -754,6 +943,9 @@ void QWaylandDisplay::handleWindowDeactivated(QWaylandWindow *window)
 
     mActiveWindows.removeOne(window);
 
+    if (QCoreApplication::closingDown())
+        return;
+
     if (auto *decoration = window->decoration())
         decoration->update();
 }
@@ -786,7 +978,7 @@ void QWaylandDisplay::handleWaylandSync()
     // handleWindowActivated() calls immediately.
     QWindow *activeWindow = mActiveWindows.empty() ? nullptr : mActiveWindows.last()->window();
     if (activeWindow != QGuiApplication::focusWindow())
-        QWindowSystemInterface::handleWindowActivated(activeWindow);
+        QWindowSystemInterface::handleFocusWindowChanged(activeWindow);
 
     if (!activeWindow) {
         if (lastInputDevice()) {
@@ -823,7 +1015,7 @@ void QWaylandDisplay::requestWaylandSync()
 
 QWaylandInputDevice *QWaylandDisplay::defaultInputDevice() const
 {
-    return mInputDevices.isEmpty() ? 0 : mInputDevices.first();
+    return mInputDevices.isEmpty() ? nullptr : mInputDevices.first();
 }
 
 bool QWaylandDisplay::isKeyboardAvailable() const
@@ -833,8 +1025,8 @@ bool QWaylandDisplay::isKeyboardAvailable() const
             [](const QWaylandInputDevice *device) { return device->keyboard() != nullptr; });
 }
 
-bool QWaylandDisplay::isClientSideInputContextRequested() const {
-    return mClientSideInputContextRequested;
+bool QWaylandDisplay::isWaylandInputContextRequested() const {
+    return mWaylandInputContextRequested;
 }
 
 #if QT_CONFIG(cursor)
