@@ -1,37 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "qwaylandquickitem.h"
 #include "qwaylandquickitem_p.h"
 #include "qwaylandquicksurface.h"
 #include "qwaylandinputmethodcontrol.h"
 #include "qwaylandtextinput.h"
+#include "qwaylandtextinputv3.h"
 #include "qwaylandqttextinputmethod.h"
 #include "qwaylandquickoutput.h"
 #include <QtWaylandCompositor/qwaylandcompositor.h>
@@ -64,7 +39,7 @@
 #include <QThread>
 
 #if QT_CONFIG(opengl)
-#include <QtGui/private/qshaderdescription_p_p.h>
+#include <QtGui/private/qshaderdescription_p.h>
 #endif
 
 #ifndef GL_TEXTURE_EXTERNAL_OES
@@ -77,65 +52,64 @@ QT_BEGIN_NAMESPACE
 static const struct {
     const char * const vertexShaderSourceFile;
     const char * const fragmentShaderSourceFile;
-    GLenum textureTarget;
     int planeCount;
     bool canProvideTexture;
     QSGMaterial::Flags materialFlags;
     QSGMaterialType materialType;
 } bufferTypes[] = {
     // BufferFormatEgl_Null
-    { "", "", 0, 0, false, {}, {} },
+    { "", "", 0, false, {}, {} },
 
-    // BufferFormatEgl_RGB
+    // BufferFormatEgl_RGB (GL_TEXTURE_2D)
     {
         ":/qt-project.org/wayland/compositor/shaders/surface.vert.qsb",
         ":/qt-project.org/wayland/compositor/shaders/surface_rgbx.frag.qsb",
-        GL_TEXTURE_2D, 1, true,
+        1, true,
         QSGMaterial::Blending,
         {}
     },
 
-    // BufferFormatEgl_RGBA
+    // BufferFormatEgl_RGBA (GL_TEXTURE_2D)
     {
         ":/qt-project.org/wayland/compositor/shaders/surface.vert.qsb",
         ":/qt-project.org/wayland/compositor/shaders/surface_rgba.frag.qsb",
-        GL_TEXTURE_2D, 1, true,
+        1, true,
         QSGMaterial::Blending,
         {}
     },
 
-    // BufferFormatEgl_EXTERNAL_OES
+    // BufferFormatEgl_EXTERNAL_OES (GL_TEXTURE_EXTERNAL_OES)
     {
         ":/qt-project.org/wayland/compositor/shaders/surface.vert.qsb",
         ":/qt-project.org/wayland/compositor/shaders/surface_oes_external.frag",
-        GL_TEXTURE_EXTERNAL_OES, 1, false,
+        1, false,
         QSGMaterial::Blending,
         {}
     },
 
-    // BufferFormatEgl_Y_U_V
+    // BufferFormatEgl_Y_U_V (GL_TEXTURE_2D)
     {
         ":/qt-project.org/wayland/compositor/shaders/surface.vert.qsb",
         ":/qt-project.org/wayland/compositor/shaders/surface_y_u_v.frag.qsb",
-        GL_TEXTURE_2D, 3, false,
+        3, false,
         QSGMaterial::Blending,
         {}
     },
 
-    // BufferFormatEgl_Y_UV
+    // BufferFormatEgl_Y_UV (GL_TEXTURE_2D)
     {
         ":/qt-project.org/wayland/compositor/shaders/surface.vert.qsb",
         ":/qt-project.org/wayland/compositor/shaders/surface_y_uv.frag.qsb",
-        GL_TEXTURE_2D, 2, false,
+        2, false,
         QSGMaterial::Blending,
         {}
     },
 
-    // BufferFormatEgl_Y_XUXV
+    // BufferFormatEgl_Y_XUXV (GL_TEXTURE_2D)
     {
         ":/qt-project.org/wayland/compositor/shaders/surface.vert.qsb",
         ":/qt-project.org/wayland/compositor/shaders/surface_y_xuxv.frag.qsb",
-        GL_TEXTURE_2D, 2, false,
+        2, false,
         QSGMaterial::Blending,
         {}
     }
@@ -183,9 +157,9 @@ void QWaylandBufferMaterialShader::setupExternalOESShader(const QString &shaderF
     descData->inVars = { texCoordInput };
 
     QShaderDescription::InOutVariable fragColorOutput;
-    texCoordInput.name = "gl_FragColor";
-    texCoordInput.type = QShaderDescription::Vec4;
-    texCoordInput.location = 0;
+    fragColorOutput.name = "gl_FragColor";
+    fragColorOutput.type = QShaderDescription::Vec4;
+    fragColorOutput.location = 0;
 
     descData->outVars = { fragColorOutput };
 
@@ -369,7 +343,7 @@ void QWaylandBufferMaterial::setBufferRef(QWaylandQuickItem *surfaceItem, const
         if (auto texture = ref.toOpenGLTexture(plane)) {
             QQuickWindow::CreateTextureOptions opt;
             QWaylandQuickSurface *waylandSurface = qobject_cast<QWaylandQuickSurface *>(surfaceItem->surface());
-            if (waylandSurface != nullptr && waylandSurface->useTextureAlpha())
+            if (waylandSurface != nullptr && waylandSurface->useTextureAlpha() && !waylandSurface->isOpaque())
                 opt |= QQuickWindow::TextureHasAlphaChannel;
             QSGTexture *scenegraphTexture;
             if (ref.bufferFormatEgl() == QWaylandBufferRef::BufferFormatEgl_EXTERNAL_OES) {
@@ -419,7 +393,7 @@ public:
 #if QT_CONFIG(opengl)
                 QQuickWindow::CreateTextureOptions opt;
                 QWaylandQuickSurface *surface = qobject_cast<QWaylandQuickSurface *>(surfaceItem->surface());
-                if (surface && surface->useTextureAlpha()) {
+                if (surface && surface->useTextureAlpha()  && !surface->isOpaque()) {
                     opt |= QQuickWindow::TextureHasAlphaChannel;
                 }
 
@@ -449,9 +423,38 @@ private:
     QWaylandBufferRef m_ref;
 };
 
+void QWaylandQuickItemPrivate::handleDragUpdate(QWaylandSeat *seat, const QPointF &globalPosition)
+{
+#if QT_CONFIG(draganddrop)
+    Q_Q(QWaylandQuickItem);
+    QWaylandQuickOutput *currentOutput = qobject_cast<QWaylandQuickOutput *>(q->view()->output());
+    //TODO: also check if dragging onto other outputs
+    QWaylandQuickItem *targetItem = qobject_cast<QWaylandQuickItem *>(currentOutput->pickClickableItem(q->mapToScene(globalPosition)));
+    QWaylandSurface *targetSurface = targetItem ? targetItem->surface() : nullptr;
+    if (targetSurface) {
+        QPointF position = q->mapToItem(targetItem, globalPosition);
+        QPointF surfacePosition = targetItem->mapToSurface(position);
+        seat->drag()->dragMove(targetSurface, surfacePosition);
+    }
+#else
+    Q_UNUSED(seat);
+    Q_UNUSED(globalPosition);
+#endif // QT_CONFIG(draganddrop)
+}
+
+void QWaylandQuickItemPrivate::handleDragEnded(QWaylandSeat *seat)
+{
+#if QT_CONFIG(draganddrop)
+    isDragging = false;
+    seat->drag()->drop();
+#else
+    Q_UNUSED(seat);
+#endif // QT_CONFIG(draganddrop)
+}
+
 /*!
  * \qmltype WaylandQuickItem
- * \instantiates QWaylandQuickItem
+ * \nativetype QWaylandQuickItem
  * \inqmlmodule QtWayland.Compositor
  * \since 5.8
  * \brief Provides a Qt Quick item that represents a WaylandView.
@@ -476,9 +479,8 @@ private:
  * Constructs a QWaylandQuickItem with the given \a parent.
  */
 QWaylandQuickItem::QWaylandQuickItem(QQuickItem *parent)
-    : QQuickItem(*new QWaylandQuickItemPrivate(), parent)
+    : QWaylandQuickItem(*new QWaylandQuickItemPrivate(), parent)
 {
-    d_func()->init();
 }
 
 /*!
@@ -488,6 +490,7 @@ QWaylandQuickItem::QWaylandQuickItem(QWaylandQuickItemPrivate &dd, QQuickItem *p
     : QQuickItem(dd, parent)
 {
     d_func()->init();
+    connect(this, &QQuickItem::activeFocusChanged, this, &QWaylandQuickItem::updateFocus);
 }
 
 /*!
@@ -497,13 +500,16 @@ QWaylandQuickItem::~QWaylandQuickItem()
 {
     Q_D(QWaylandQuickItem);
     disconnect(this, &QQuickItem::windowChanged, this, &QWaylandQuickItem::updateWindow);
+    disconnect(this, &QQuickItem::activeFocusChanged, this, &QWaylandQuickItem::updateFocus);
     QMutexLocker locker(d->mutex);
-    if (d->provider)
+    if (d->provider) {
+        disconnect(d->texProviderConnection);
         d->provider->deleteLater();
+    }
 }
 
 /*!
- * \qmlproperty WaylandCompositor QtWaylandCompositor::WaylandQuickItem::compositor
+ * \qmlproperty WaylandCompositor QtWayland.Compositor::WaylandQuickItem::compositor
  *
  * This property holds the compositor for the surface rendered by this WaylandQuickItem.
  */
@@ -529,7 +535,7 @@ QWaylandView *QWaylandQuickItem::view() const
 }
 
 /*!
- * \qmlproperty WaylandSurface QtWaylandCompositor::WaylandQuickItem::surface
+ * \qmlproperty WaylandSurface QtWayland.Compositor::WaylandQuickItem::surface
  *
  * This property holds the surface rendered by this WaylandQuickItem.
  */
@@ -557,11 +563,13 @@ void QWaylandQuickItem::setSurface(QWaylandSurface *surface)
         emit compositorChanged();
     if (oldSurf != surface)
         emit surfaceChanged();
+
+    updateFocus();
     update();
 }
 
 /*!
- * \qmlproperty enum QtWaylandCompositor::WaylandQuickItem::origin
+ * \qmlproperty enum QtWayland.Compositor::WaylandQuickItem::origin
  *
  * This property holds the origin of the QWaylandQuickItem.
  */
@@ -630,20 +638,9 @@ void QWaylandQuickItem::mouseMoveEvent(QMouseEvent *event)
     Q_D(QWaylandQuickItem);
     if (d->shouldSendInputEvents()) {
         QWaylandSeat *seat = compositor()->seatFor(event);
-#if QT_CONFIG(draganddrop)
         if (d->isDragging) {
-            QWaylandQuickOutput *currentOutput = qobject_cast<QWaylandQuickOutput *>(view()->output());
-            //TODO: also check if dragging onto other outputs
-            QWaylandQuickItem *targetItem = qobject_cast<QWaylandQuickItem *>(currentOutput->pickClickableItem(mapToScene(event->position())));
-            QWaylandSurface *targetSurface = targetItem ? targetItem->surface() : nullptr;
-            if (targetSurface) {
-                QPointF position = mapToItem(targetItem, event->position());
-                QPointF surfacePosition = targetItem->mapToSurface(position);
-                seat->drag()->dragMove(targetSurface, surfacePosition);
-            }
-        } else
-#endif // QT_CONFIG(draganddrop)
-        {
+            d->handleDragUpdate(seat, event->position());
+        } else {
             seat->sendMouseMoveEvent(d->view.data(), mapToSurface(event->position()), event->scenePosition());
             d->hoverPos = event->position();
         }
@@ -661,15 +658,10 @@ void QWaylandQuickItem::mouseReleaseEvent(QMouseEvent *event)
     Q_D(QWaylandQuickItem);
     if (d->shouldSendInputEvents()) {
         QWaylandSeat *seat = compositor()->seatFor(event);
-#if QT_CONFIG(draganddrop)
-        if (d->isDragging) {
-            d->isDragging = false;
-            seat->drag()->drop();
-        } else
-#endif
-        {
+        if (d->isDragging)
+            d->handleDragEnded(seat);
+        else
             seat->sendMouseReleaseEvent(event->button());
-        }
     } else {
         emit mouseRelease();
         event->ignore();
@@ -780,7 +772,7 @@ void QWaylandQuickItem::keyPressEvent(QKeyEvent *event)
 void QWaylandQuickItem::keyReleaseEvent(QKeyEvent *event)
 {
     Q_D(QWaylandQuickItem);
-    if (d->shouldSendInputEvents() && hasFocus()) {
+    if (d->shouldSendInputEvents()) {
         QWaylandSeat *seat = compositor()->seatFor(event);
         seat->sendFullKeyEvent(event);
     } else {
@@ -807,6 +799,9 @@ void QWaylandQuickItem::touchEvent(QTouchEvent *event)
             return;
         }
 
+        if (event->type() == QEvent::TouchUpdate && d->isDragging)
+            d->handleDragUpdate(seat, pointPos);
+
         event->accept();
         if (seat->mouseFocus() != d->view.data()) {
             seat->sendMouseMoveEvent(d->view.data(), pointPos, mapToScene(pointPos));
@@ -816,6 +811,8 @@ void QWaylandQuickItem::touchEvent(QTouchEvent *event)
         if (event->type() == QEvent::TouchBegin) {
             d->touchingSeats.append(seat);
         } else if (event->type() == QEvent::TouchEnd || event->type() == QEvent::TouchCancel) {
+            if (d->isDragging)
+                d->handleDragEnded(seat);
             d->touchingSeats.removeOne(seat);
         }
 
@@ -874,6 +871,10 @@ void QWaylandQuickItem::handleSubsurfaceAdded(QWaylandSurface *childSurface)
         connect(childSurface, &QWaylandSurface::destroyed, childItem, &QObject::deleteLater);
     } else {
         bool success = QMetaObject::invokeMethod(d->subsurfaceHandler, "handleSubsurfaceAdded", Q_ARG(QWaylandSurface *, childSurface));
+        if (!success)
+            success = QMetaObject::invokeMethod(d->subsurfaceHandler, "handleSubsurfaceAdded",
+                                                Q_ARG(QVariant, QVariant::fromValue(childSurface)));
+
         if (!success)
             qWarning("QWaylandQuickItem: subsurfaceHandler does not implement handleSubsurfaceAdded()");
     }
@@ -913,8 +914,15 @@ void QWaylandQuickItem::handlePlaceBelow(QWaylandSurface *referenceSurface)
     }
 }
 
+void QWaylandQuickItem::updateFocus()
+{
+    Q_D(const QWaylandQuickItem);
+    if (hasActiveFocus() && compositor())
+        compositor()->defaultSeat()->setKeyboardFocus(d->view->surface());
+}
+
 /*!
-  \qmlproperty object QtWaylandCompositor::WaylandQuickItem::subsurfaceHandler
+  \qmlproperty object QtWayland.Compositor::WaylandQuickItem::subsurfaceHandler
 
   This property provides a way to override the default subsurface behavior.
 
@@ -926,8 +934,9 @@ void QWaylandQuickItem::handlePlaceBelow(QWaylandSurface *referenceSurface)
   \code
   ShellSurfaceItem {
       subsurfaceHandler: QtObject {
-      function handleSubsurfaceAdded(child) {
-        //create custom surface item, and connect the subsurfacePositionChanged signal
+          function handleSubsurfaceAdded(child) {
+            // create custom surface item, and connect the subsurfacePositionChanged signal
+          }
       }
   }
   \endcode
@@ -951,6 +960,11 @@ void QWaylandQuickItem::setSubsurfaceHandler(QObject *handler)
     }
 }
 
+/*!
+ * \qmlproperty WaylandOutput QtWayland.Compositor::WaylandQuickItem::output
+ *
+ * This property holds the output on which this item is displayed.
+ */
 /*!
  * \property QWaylandQuickItem::output
  *
@@ -969,7 +983,7 @@ void QWaylandQuickItem::setOutput(QWaylandOutput *output)
 }
 
 /*!
- * \qmlproperty bool QtWaylandCompositor::WaylandQuickItem::bufferLocked
+ * \qmlproperty bool QtWayland.Compositor::WaylandQuickItem::bufferLocked
  *
  * This property holds whether the item's buffer is currently locked. As long as
  * the buffer is locked, it will not be released and returned to the client.
@@ -1107,7 +1121,7 @@ void QWaylandQuickItem::takeFocus(QWaylandSeat *device)
 {
     forceActiveFocus();
 
-    if (!surface())
+    if (!surface() || !surface()->client())
         return;
 
     QWaylandSeat *target = device;
@@ -1116,13 +1130,22 @@ void QWaylandQuickItem::takeFocus(QWaylandSeat *device)
     }
     target->setKeyboardFocus(surface());
 
-    {
+    qCDebug(qLcWaylandCompositorInputMethods) << Q_FUNC_INFO << " surface:" << surface()
+        << ", client:" << surface()->client()
+        << ", textinputprotocol:" << (int)(surface()->client()->textInputProtocols());
+    if (surface()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::TextInputV2)) {
         QWaylandTextInput *textInput = QWaylandTextInput::findIn(target);
         if (textInput)
             textInput->setFocus(surface());
     }
 
-    {
+    if (surface()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::TextInputV3)) {
+        QWaylandTextInputV3 *textInputV3 = QWaylandTextInputV3::findIn(target);
+        if (textInputV3)
+            textInputV3->setFocus(surface());
+    }
+
+    if (surface()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::QtTextInputMethodV1)) {
         QWaylandQtTextInputMethod *textInputMethod = QWaylandQtTextInputMethod::findIn(target);
         if (textInputMethod)
             textInputMethod->setFocus(surface());
@@ -1173,7 +1196,7 @@ void QWaylandQuickItem::updateSize()
 }
 
 /*!
- * \qmlproperty bool QtWaylandCompositor::WaylandQuickItem::focusOnClick
+ * \qmlproperty bool QtWayland.Compositor::WaylandQuickItem::focusOnClick
  *
  * This property specifies whether the WaylandQuickItem should take focus when
  * it is clicked or touched.
@@ -1288,7 +1311,7 @@ QVariant QWaylandQuickItem::inputMethodQuery(Qt::InputMethodQuery query, QVarian
 #endif
 
 /*!
-    \qmlproperty bool QtWaylandCompositor::WaylandQuickItem::paintEnabled
+    \qmlproperty bool QtWayland.Compositor::WaylandQuickItem::paintEnabled
 
     Returns true if the item is hidden, though the texture
     is still updated. As opposed to hiding the item by
@@ -1323,7 +1346,7 @@ void QWaylandQuickItem::setPaintEnabled(bool enabled)
 }
 
 /*!
-    \qmlproperty  bool QtWaylandCompositor::WaylandQuickItem::touchEventsEnabled
+    \qmlproperty  bool QtWayland.Compositor::WaylandQuickItem::touchEventsEnabled
 
     This property holds \c true if touch events are forwarded to the client
     surface, \c false otherwise.
@@ -1418,7 +1441,7 @@ void QWaylandQuickItem::updateInputMethod(Qt::InputMethodQueries queries)
 #endif
 
 /*!
- * \qmlsignal void QtWaylandCompositor::WaylandQuickItem::surfaceDestroyed()
+ * \qmlsignal void QtWayland.Compositor::WaylandQuickItem::surfaceDestroyed()
  *
  * This signal is emitted when the client has destroyed the \c wl_surface associated
  * with the WaylandQuickItem. The handler for this signal is expected to either destroy the
@@ -1485,8 +1508,24 @@ QSGNode *QWaylandQuickItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
             d->newTexture = true;
         }
 
-        if (!d->provider)
+        if (!d->provider) {
             d->provider = new QWaylandSurfaceTextureProvider();
+            if (compositor()) {
+                d->texProviderConnection =
+                    QObject::connect(
+                            compositor(),
+                            &QObject::destroyed,
+                            this,
+                            [this](QObject*) {
+                                    auto *itemPriv = QWaylandQuickItemPrivate::get(this);
+                                    if (itemPriv->provider) {
+                                        itemPriv->provider->deleteLater();
+                                        itemPriv->provider = nullptr;
+                                    }
+                                    disconnect(itemPriv->texProviderConnection); }
+                    );
+            }
+        }
 
         if (d->newTexture) {
             d->newTexture = false;
@@ -1588,20 +1627,34 @@ void QWaylandQuickItem::setInputEventsEnabled(bool enabled)
 
 void QWaylandQuickItem::lower()
 {
-    QQuickItem *parent = parentItem();
+    Q_D(QWaylandQuickItem);
+    d->lower();
+}
+
+void QWaylandQuickItemPrivate::lower()
+{
+    Q_Q(QWaylandQuickItem);
+    QQuickItem *parent = q->parentItem();
     Q_ASSERT(parent);
-    QQuickItem *bottom = parent->childItems().first();
-    if (this != bottom)
-        stackBefore(bottom);
+    QQuickItem *bottom = parent->childItems().constFirst();
+    if (q != bottom)
+        q->stackBefore(bottom);
 }
 
 void QWaylandQuickItem::raise()
 {
-    QQuickItem *parent = parentItem();
+    Q_D(QWaylandQuickItem);
+    d->raise();
+}
+
+void QWaylandQuickItemPrivate::raise()
+{
+    Q_Q(QWaylandQuickItem);
+    QQuickItem *parent = q->parentItem();
     Q_ASSERT(parent);
-    QQuickItem *top = parent->childItems().last();
-    if (this != top)
-        stackAfter(top);
+    QQuickItem *top = parent->childItems().constLast();
+    if (q != top)
+        q->stackAfter(top);
 }
 
 void QWaylandQuickItem::sendMouseMoveEvent(const QPointF &position, QWaylandSeat *seat)
@@ -1731,3 +1784,5 @@ void QWaylandQuickItemPrivate::placeBelowParent()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qwaylandquickitem.cpp"
