@@ -1,9 +1,8 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "mockcompositor.h"
 
-#include <QtOpenGL/QOpenGLWindow>
 #include <QtGui/QRasterWindow>
 #if QT_CONFIG(cursor)
 #include <wayland-cursor.h>
@@ -24,6 +23,7 @@ public:
     {
         exec([this] {
             m_config.autoConfigure = true;
+            m_config.autoFrameCallback = false; // for cursor animation test
 
             removeAll<Seat>();
 
