@@ -1,5 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef MOCKCOMPOSITOR_CORECOMPOSITOR_H
 #define MOCKCOMPOSITOR_CORECOMPOSITOR_H
@@ -29,7 +29,8 @@ public:
     };
 
     CompositorType m_type = Default;
-    explicit CoreCompositor(CompositorType t = Default);
+    explicit CoreCompositor(CompositorType t = Default, int socketFd = -1);
+
     ~CoreCompositor();
     bool isClean();
     QString dirtyMessage();
@@ -178,7 +179,6 @@ protected:
         CoreCompositor *m_compositor = nullptr;
         std::thread::id m_threadId;
     };
-    QByteArray m_socketName;
     wl_event_loop *m_eventLoop = nullptr;
     bool m_running = true;
     QList<Global *> m_globals;
