@@ -1,5 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "xdgshell.h"
 
@@ -143,6 +143,11 @@ XdgToplevel::XdgToplevel(XdgSurface *xdgSurface, int id, int version)
     connect(surface(), &Surface::commit, this, [this] { m_committed = m_pending; });
 }
 
+void XdgToplevel::sendConfigureBounds(const QSize &size)
+{
+    send_configure_bounds(size.width(), size.height());
+}
+
 void XdgToplevel::sendConfigure(const QSize &size, const QList<uint> &states)
 {
     send_configure(size.width(), size.height(), toByteArray(states));
